import axios from 'axios'
import MarkdownIt from 'markdown-it'
import { useState } from 'react'
import { Helmet } from 'react-helmet-async'
import MdEditor from 'react-markdown-editor-lite'
import { useLocation, useNavigate } from 'react-router-dom'

import MainArea from '@/components/layout/MainArea'
import { toast } from '@/components/ui/use-toast'
import { API_BASE_URL, SITE_TITLE } from '@/config'
import Forbidden from '@/pages/Forbidden'

import 'react-markdown-editor-lite/lib/index.css'

import type { User, WikiPage } from '@/types'

const mdParser = new MarkdownIt

type Props = { user: User | null }


export default ({ user }: Props) => {
  if (!(['admin', 'member'].some (r => user?.role === r)))
    return <Forbidden />

  const location = useLocation ()
  const navigate = useNavigate ()

  const query = new URLSearchParams (location.search)
  const titleQuery = query.get ('title') ?? ''

  const [title, setTitle] = useState (titleQuery)
  const [body, setBody] = useState ('')

  const handleSubmit = async () => {
    const formData = new FormData
    formData.append ('title', title)
    formData.append ('body', body)

    try
    {
      const res = await axios.post (`${ API_BASE_URL }/wiki`, formData, { headers: {
	  'Content-Type':     'multipart/form-data',
	  'X-Transfer-Code':  localStorage.getItem ('user_code') || '' } })
      const data = res.data as WikiPage
      toast ({ title: '投稿成功!' })
      navigate (`/wiki/${ data.title }`)
    }
    catch
    {
      toast ({ title: '投稿失敗', description: '入力を確認してください。' })
    }
  }

  return (
      <MainArea>
	<Helmet>
	  <title>{`新規 Wiki ページ | ${ SITE_TITLE }`}</title>
	</Helmet>
	<div className="max-w-xl mx-auto p-4 space-y-4">
	  <h1 className="text-2xl font-bold mb-2">新規 Wiki ページ</h1>

	  {/* タイトル */}
	  {/* TODO: タグ補完 */}
	  <div>
	    <label className="block font-semibold mb-1">タイトル</label>
	    <input type="text"
		   value={title}
		   onChange={e => setTitle (e.target.value)}
		   className="w-full border p-2 rounded" />
	  </div>

	  {/* 本文 */}
	  <div>
	    <label className="block font-semibold mb-1">本文</label>
	    <MdEditor value={body}
		      style={{ height: '500px' }}
		      renderHTML={text => mdParser.render (text)}
		      onChange={({ text }) => setBody (text)} />
	  </div>

	  {/* 送信 */}
	  <button onClick={handleSubmit}
		  className="px-4 py-2 bg-blue-600 text-white rounded disabled:bg-gray-400">
	    追加
	  </button>
	</div>
      </MainArea>)
}
